package java.util;

import java.util.function.UnaryOperator;
import java.util.function.Predicate;
import java.util.function.Consumer;
import sun.misc.SharedSecrets;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

public class ArrayList<E> extends AbstractList<E> implements List<E>, RandomAccess, Cloneable, Serializable
{
    private static final long serialVersionUID = 8683452581122892189L;
    private static final int DEFAULT_CAPACITY = 10;
    private static final Object[] EMPTY_ELEMENTDATA;
    private static final Object[] DEFAULTCAPACITY_EMPTY_ELEMENTDATA;
    transient Object[] elementData;
    private int size;
    private static final int MAX_ARRAY_SIZE = 2147483639;
    
    public ArrayList(int i) {
        if (i > 0) {
            this.elementData = new Object[i];
        }
        else if (i == 0) {
            this.elementData = ArrayList.EMPTY_ELEMENTDATA;
        }
        else {
            throw new IllegalArgumentException("Illegal Capacity: " + i);
        }
    }
    
    public ArrayList() {
        this.elementData = ArrayList.DEFAULTCAPACITY_EMPTY_ELEMENTDATA;
    }
    
    public ArrayList(Collection<? extends E> collection) {
        Object[] array = collection.toArray();
        int length = array.length;
        this.size = length;
        if (length != 0) {
            if (collection.getClass() == ArrayList.class) {
                this.elementData = array;
            }
            else {
                this.elementData = Arrays.copyOf(array, this.size, Object[].class);
            }
        }
        else {
            this.elementData = ArrayList.EMPTY_ELEMENTDATA;
        }
    }
    
    public void trimToSize() {
        ++this.modCount;
        if (this.size < this.elementData.length) {
            this.elementData = ((this.size == 0) ? ArrayList.EMPTY_ELEMENTDATA : Arrays.copyOf(this.elementData, this.size));
        }
    }
    
    public void ensureCapacity(int n) {
        if (n > ((this.elementData != ArrayList.DEFAULTCAPACITY_EMPTY_ELEMENTDATA) ? 0 : 10)) {
            this.ensureExplicitCapacity(n);
        }
    }
    
    private static int calculateCapacity(Object[] array, int b) {
        if (array == ArrayList.DEFAULTCAPACITY_EMPTY_ELEMENTDATA) {
            return Math.max(10, b);
        }
        return b;
    }
    
    private void ensureCapacityInternal(int n) {
        this.ensureExplicitCapacity(calculateCapacity(this.elementData, n));
    }
    
    private void ensureExplicitCapacity(int n) {
        ++this.modCount;
        if (n - this.elementData.length > 0) {
            this.grow(n);
        }
    }
    
    private void grow(int n) {
        int length = this.elementData.length;
        int hugeCapacity = length + (length >> 1);
        if (hugeCapacity - n < 0) {
            hugeCapacity = n;
        }
        if (hugeCapacity - 2147483639 > 0) {
            hugeCapacity = hugeCapacity(n);
        }
        this.elementData = Arrays.copyOf(this.elementData, hugeCapacity);
    }
    
    private static int hugeCapacity(int n) {
        if (n < 0) {
            throw new OutOfMemoryError();
        }
        return (n > 2147483639) ? Integer.MAX_VALUE : 2147483639;
    }
    
    @Override
    public int size() {
        return this.size;
    }
    
    @Override
    public boolean isEmpty() {
        return this.size == 0;
    }
    
    @Override
    public boolean contains(Object o) {
        return this.indexOf(o) >= 0;
    }
    
    @Override
    public int indexOf(Object o) {
        if (o == null) {
            int j = 0;
            while (j < this.size) {
                if (this.elementData[j] == null) {
                    return j;
                }
                ++j;
            }
        }
        else {
            int i = 0;
            while (i < this.size) {
                if (o.equals(this.elementData[i])) {
                    return i;
                }
                ++i;
            }
        }
        return -1;
    }
    
    @Override
    public int lastIndexOf(Object o) {
        if (o == null) {
            int j = this.size - 1;
            while (j >= 0) {
                if (this.elementData[j] == null) {
                    return j;
                }
                --j;
            }
        }
        else {
            int i = this.size - 1;
            while (i >= 0) {
                if (o.equals(this.elementData[i])) {
                    return i;
                }
                --i;
            }
        }
        return -1;
    }
    
    public Object clone() {
        try {
            ArrayList<Object> list = (ArrayList)super.clone();
            list.elementData = Arrays.copyOf(this.elementData, this.size);
            list.modCount = 0;
            return list;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    @Override
    public Object[] toArray() {
        return Arrays.copyOf(this.elementData, this.size);
    }
    
    @Override
    public <T> T[] toArray(T[] array) {
        if (array.length < this.size) {
            return (T[])Arrays.copyOf(this.elementData, this.size, array.getClass());
        }
        System.arraycopy(this.elementData, 0, array, 0, this.size);
        if (array.length > this.size) {
            array[this.size] = null;
            return array;
        }
        return array;
    }
    
    E elementData(int n) {
        return this.elementData[n];
    }
    
    @Override
    public E get(int n) {
        this.rangeCheck(n);
        return this.elementData(n);
    }
    
    @Override
    public E set(int n, E e) {
        this.rangeCheck(n);
        Object elementData = this.elementData(n);
        this.elementData[n] = e;
        return elementData;
    }
    
    @Override
    public boolean add(E e) {
        this.ensureCapacityInternal(this.size + 1);
        this.elementData[this.size++] = e;
        return true;
    }
    
    @Override
    public void add(int n, E e) {
        this.rangeCheckForAdd(n);
        this.ensureCapacityInternal(this.size + 1);
        System.arraycopy(this.elementData, n, this.elementData, n + 1, this.size - n);
        this.elementData[n] = e;
        ++this.size;
    }
    
    @Override
    public E remove(int n) {
        this.rangeCheck(n);
        ++this.modCount;
        Object elementData = this.elementData(n);
        int n2 = this.size - n - 1;
        if (n2 > 0) {
            System.arraycopy(this.elementData, n + 1, this.elementData, n, n2);
        }
        this.elementData[--this.size] = null;
        return elementData;
    }
    
    @Override
    public boolean remove(Object o) {
        if (o == null) {
            int j = 0;
            while (j < this.size) {
                if (this.elementData[j] == null) {
                    this.fastRemove(j);
                    return true;
                }
                ++j;
            }
        }
        else {
            int i = 0;
            while (i < this.size) {
                if (o.equals(this.elementData[i])) {
                    this.fastRemove(i);
                    return true;
                }
                ++i;
            }
        }
        return false;
    }
    
    private void fastRemove(int n) {
        ++this.modCount;
        int n2 = this.size - n - 1;
        if (n2 > 0) {
            System.arraycopy(this.elementData, n + 1, this.elementData, n, n2);
        }
        this.elementData[--this.size] = null;
    }
    
    @Override
    public void clear() {
        ++this.modCount;
        for (int i = 0; i < this.size; ++i) {
            this.elementData[i] = null;
        }
        this.size = 0;
    }
    
    @Override
    public boolean addAll(Collection<? extends E> collection) {
        Object[] array = collection.toArray();
        int length = array.length;
        this.ensureCapacityInternal(this.size + length);
        System.arraycopy(array, 0, this.elementData, this.size, length);
        this.size += length;
        return length != 0;
    }
    
    @Override
    public boolean addAll(int n, Collection<? extends E> collection) {
        this.rangeCheckForAdd(n);
        Object[] array = collection.toArray();
        int length = array.length;
        this.ensureCapacityInternal(this.size + length);
        int n2 = this.size - n;
        if (n2 > 0) {
            System.arraycopy(this.elementData, n, this.elementData, n + length, n2);
        }
        System.arraycopy(array, 0, this.elementData, n, length);
        this.size += length;
        return length != 0;
    }
    
    @Override
    protected void removeRange(int n, int n2) {
        ++this.modCount;
        System.arraycopy(this.elementData, n2, this.elementData, n, this.size - n2);
        int i;
        for (i = (i = this.size - (n2 - n)); i < this.size; ++i) {
            this.elementData[i] = null;
        }
        this.size = i;
    }
    
    private void rangeCheck(int n) {
        if (n >= this.size) {
            throw new IndexOutOfBoundsException(this.outOfBoundsMsg(n));
        }
        return;
    }
    
    private void rangeCheckForAdd(int n) {
        if (n > this.size || n < 0) {
            throw new IndexOutOfBoundsException(this.outOfBoundsMsg(n));
        }
        return;
    }
    
    private String outOfBoundsMsg(int i) {
        return "Index: " + i + ", Size: " + this.size;
    }
    
    @Override
    public boolean removeAll(Collection<?> collection) {
        Objects.requireNonNull(collection);
        return this.batchRemove(collection, false);
    }
    
    @Override
    public boolean retainAll(Collection<?> collection) {
        Objects.requireNonNull(collection);
        return this.batchRemove(collection, true);
    }
    
    private boolean batchRemove(Collection<?> collection, boolean b) {
        Object[] elementData = this.elementData;
        int i = 0;
        int j = 0;
        boolean n = false;
        try {
            while (i < this.size) {
                if (collection.contains(elementData[i]) == b) {
                    elementData[j++] = elementData[i];
                }
                ++i;
            }
        }
        finally {
            if (i != this.size) {
                System.arraycopy(elementData, i, elementData, j, this.size - i);
                j += this.size - i;
            }
            if (j != this.size) {
                for (; j < this.size; ++j) {
                    elementData[j] = null;
                }
                this.modCount += this.size - j;
                this.size = j;
                n = true;
            }
        }
        return n;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        int modCount = this.modCount;
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(this.size);
        for (int i = 0; i < this.size; ++i) {
            objectOutputStream.writeObject(this.elementData[i]);
        }
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        return;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        this.elementData = ArrayList.EMPTY_ELEMENTDATA;
        objectInputStream.defaultReadObject();
        objectInputStream.readInt();
        if (this.size > 0) {
            SharedSecrets.getJavaOISAccess().checkArray(objectInputStream, Object[].class, calculateCapacity(this.elementData, this.size));
            this.ensureCapacityInternal(this.size);
            Object[] elementData = this.elementData;
            for (int i = 0; i < this.size; ++i) {
                elementData[i] = objectInputStream.readObject();
            }
        }
    }
    
    @Override
    public ListIterator<E> listIterator(int i) {
        if (i < 0 || i > this.size) {
            throw new IndexOutOfBoundsException("Index: " + i);
        }
        return new ArrayList.ListItr(i);
    }
    
    @Override
    public ListIterator<E> listIterator() {
        return new ArrayList.ListItr(0);
    }
    
    @Override
    public Iterator<E> iterator() {
        return new ArrayList.Itr();
    }
    
    @Override
    public List<E> subList(int n, int n2) {
        subListRangeCheck(n, n2, this.size);
        return new ArrayList.SubList(this, 0, n, n2);
    }
    
    static void subListRangeCheck(int n, int n2, int n3) {
        if (n < 0) {
            throw new IndexOutOfBoundsException("fromIndex = " + n);
        }
        if (n2 > n3) {
            throw new IndexOutOfBoundsException("toIndex = " + n2);
        }
        if (n > n2) {
            throw new IllegalArgumentException("fromIndex(" + n + ") > toIndex(" + n2 + ")");
        }
        return;
    }
    
    @Override
    public void forEach(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        int modCount = this.modCount;
        Object[] array = (Object[])this.elementData;
        for (int size = this.size, n = 0; this.modCount == modCount && n < size; ++n) {
            consumer.accept(array[n]);
        }
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        return;
    }
    
    @Override
    public Spliterator<E> spliterator() {
        return new ArrayList.ArrayListSpliterator(this, 0, -1, 0);
    }
    
    @Override
    public boolean removeIf(Predicate<? super E> predicate) {
        Objects.requireNonNull(predicate);
        int n = 0;
        BitSet set = new BitSet(this.size);
        int modCount = this.modCount;
        int size = this.size;
        for (int n2 = 0; this.modCount == modCount && n2 < size; ++n2) {
            if (predicate.test(this.elementData[n2])) {
                set.set(n2);
                ++n;
            }
        }
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        int n3 = n > 0;
        if (!(n3 == 0)) {
            int i = size - n;
            for (int nextClearBit = 0, n4 = 0; nextClearBit < size && n4 < i; nextClearBit = set.nextClearBit(nextClearBit), this.elementData[n4] = this.elementData[nextClearBit], ++nextClearBit, ++n4) {}
            for (; i < size; ++i) {
                this.elementData[i] = null;
            }
            this.size = i;
            if (this.modCount != modCount) {
                throw new ConcurrentModificationException();
            }
            ++this.modCount;
        }
        return n3;
    }
    
    @Override
    public void replaceAll(UnaryOperator<E> unaryOperator) {
        Objects.requireNonNull(unaryOperator);
        int modCount = this.modCount;
        for (int size = this.size, n = 0; this.modCount == modCount && n < size; ++n) {
            this.elementData[n] = unaryOperator.apply(this.elementData[n]);
        }
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        ++this.modCount;
        return;
    }
    
    @Override
    public void sort(Comparator<? super E> comparator) {
        int modCount = this.modCount;
        Arrays.sort((Object[])this.elementData, 0, this.size, comparator);
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        ++this.modCount;
        return;
    }
    
    static {
        EMPTY_ELEMENTDATA = new Object[0];
        DEFAULTCAPACITY_EMPTY_ELEMENTDATA = new Object[0];
    }
    
    private class Itr implements Iterator<E>
    {
        int cursor;
        int lastRet;
        int expectedModCount;
        
        Itr() {
            this.lastRet = -1;
            this.expectedModCount = ArrayList.this.modCount;
        }
        
        @Override
        public boolean hasNext() {
            return this.cursor != ArrayList.this.size;
        }
        
        @Override
        public E next() {
            this.checkForComodification();
            int cursor = this.cursor;
            if (cursor >= ArrayList.this.size) {
                throw new NoSuchElementException();
            }
            Object[] elementData = ArrayList.this.elementData;
            if (cursor >= elementData.length) {
                throw new ConcurrentModificationException();
            }
            this.cursor = cursor + 1;
            Object[] array = elementData;
            int lastRet = cursor;
            this.lastRet = lastRet;
            return array[lastRet];
        }
        
        @Override
        public void remove() {
            if (this.lastRet < 0) {
                throw new IllegalStateException();
            }
            this.checkForComodification();
            try {
                ArrayList.this.remove(this.lastRet);
                this.cursor = this.lastRet;
                this.lastRet = -1;
                this.expectedModCount = ArrayList.this.modCount;
            }
            catch (IndexOutOfBoundsException ex) {
                throw new ConcurrentModificationException();
            }
            return;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super E> consumer) {
            Objects.requireNonNull(consumer);
            int access$000 = ArrayList.this.size;
            int cursor = this.cursor;
            if (cursor >= access$000) {
                return;
            }
            Object[] elementData = ArrayList.this.elementData;
            if (cursor >= elementData.length) {
                throw new ConcurrentModificationException();
            }
            while (cursor != access$000 && ArrayList.this.modCount == this.expectedModCount) {
                consumer.accept(elementData[cursor++]);
            }
            this.cursor = cursor;
            this.lastRet = cursor - 1;
            this.checkForComodification();
            return;
        }
        
        final void checkForComodification() {
            if (ArrayList.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return;
        }
    }
    
    private class ListItr extends ArrayList.Itr implements ListIterator<E>
    {
        ListItr(int cursor) {
            this.cursor = cursor;
        }
        
        @Override
        public boolean hasPrevious() {
            return this.cursor != 0;
        }
        
        @Override
        public int nextIndex() {
            return this.cursor;
        }
        
        @Override
        public int previousIndex() {
            return this.cursor - 1;
        }
        
        @Override
        public E previous() {
            this.checkForComodification();
            int cursor = this.cursor - 1;
            if (cursor < 0) {
                throw new NoSuchElementException();
            }
            Object[] elementData = ArrayList.this.elementData;
            if (cursor >= elementData.length) {
                throw new ConcurrentModificationException();
            }
            this.cursor = cursor;
            Object[] array = elementData;
            int lastRet = cursor;
            this.lastRet = lastRet;
            return array[lastRet];
        }
        
        @Override
        public void set(E e) {
            if (this.lastRet < 0) {
                throw new IllegalStateException();
            }
            this.checkForComodification();
            try {
                ArrayList.this.set(this.lastRet, e);
            }
            catch (IndexOutOfBoundsException ex) {
                throw new ConcurrentModificationException();
            }
            return;
        }
        
        @Override
        public void add(E e) {
            this.checkForComodification();
            try {
                int cursor = this.cursor;
                ArrayList.this.add(cursor, e);
                this.cursor = cursor + 1;
                this.lastRet = -1;
                this.expectedModCount = ArrayList.this.modCount;
            }
            catch (IndexOutOfBoundsException ex) {
                throw new ConcurrentModificationException();
            }
        }
    }
    
    private class SubList extends AbstractList<E> implements RandomAccess
    {
        private final AbstractList<E> parent;
        private final int parentOffset;
        private final int offset;
        int size;
        
        SubList(AbstractList<E> parent, int n, int parentOffset, int n2) {
            this.parent = parent;
            this.parentOffset = parentOffset;
            this.offset = n + parentOffset;
            this.size = n2 - parentOffset;
            this.modCount = ArrayList.this.modCount;
        }
        
        @Override
        public E set(int n, E e) {
            this.rangeCheck(n);
            this.checkForComodification();
            Object elementData = ArrayList.this.elementData(this.offset + n);
            ArrayList.this.elementData[this.offset + n] = e;
            return elementData;
        }
        
        @Override
        public E get(int n) {
            this.rangeCheck(n);
            this.checkForComodification();
            return ArrayList.this.elementData(this.offset + n);
        }
        
        @Override
        public int size() {
            this.checkForComodification();
            return this.size;
        }
        
        @Override
        public void add(int n, E e) {
            this.rangeCheckForAdd(n);
            this.checkForComodification();
            this.parent.add(this.parentOffset + n, e);
            this.modCount = this.parent.modCount;
            ++this.size;
        }
        
        @Override
        public E remove(int n) {
            this.rangeCheck(n);
            this.checkForComodification();
            Object remove = this.parent.remove(this.parentOffset + n);
            this.modCount = this.parent.modCount;
            --this.size;
            return remove;
        }
        
        @Override
        protected void removeRange(int n, int n2) {
            this.checkForComodification();
            this.parent.removeRange(this.parentOffset + n, this.parentOffset + n2);
            this.modCount = this.parent.modCount;
            this.size -= n2 - n;
        }
        
        @Override
        public boolean addAll(Collection<? extends E> collection) {
            return this.addAll(this.size, collection);
        }
        
        @Override
        public boolean addAll(int n, Collection<? extends E> collection) {
            this.rangeCheckForAdd(n);
            int size = collection.size();
            if (size == 0) {
                return false;
            }
            this.checkForComodification();
            this.parent.addAll(this.parentOffset + n, collection);
            this.modCount = this.parent.modCount;
            this.size += size;
            return true;
        }
        
        @Override
        public Iterator<E> iterator() {
            return super.listIterator();
        }
        
        @Override
        public ListIterator<E> listIterator(int n) {
            this.checkForComodification();
            this.rangeCheckForAdd(n);
            return new ListIterator<E>() {
                int cursor = n;
                int lastRet = -1;
                int expectedModCount = ArrayList.this.modCount;
                
                @Override
                public boolean hasNext() {
                    return this.cursor != ArrayList.SubList.this.size;
                }
                
                @Override
                public E next() {
                    this.checkForComodification();
                    int cursor = this.cursor;
                    if (cursor >= ArrayList.SubList.this.size) {
                        throw new NoSuchElementException();
                    }
                    Object[] elementData = ArrayList.this.elementData;
                    if (offset + cursor >= elementData.length) {
                        throw new ConcurrentModificationException();
                    }
                    this.cursor = cursor + 1;
                    Object[] array = elementData;
                    int val$offset = offset;
                    int lastRet = cursor;
                    this.lastRet = lastRet;
                    return array[offset + lastRet];
                }
                
                @Override
                public boolean hasPrevious() {
                    return this.cursor != 0;
                }
                
                @Override
                public E previous() {
                    this.checkForComodification();
                    int cursor = this.cursor - 1;
                    if (cursor < 0) {
                        throw new NoSuchElementException();
                    }
                    Object[] elementData = ArrayList.this.elementData;
                    if (offset + cursor >= elementData.length) {
                        throw new ConcurrentModificationException();
                    }
                    this.cursor = cursor;
                    Object[] array = elementData;
                    int val$offset = offset;
                    int lastRet = cursor;
                    this.lastRet = lastRet;
                    return array[offset + lastRet];
                }
                
                @Override
                public void forEachRemaining(Consumer<? super E> consumer) {
                    Objects.requireNonNull(consumer);
                    int size = ArrayList.SubList.this.size;
                    int cursor = this.cursor;
                    if (cursor >= size) {
                        return;
                    }
                    Object[] elementData = ArrayList.this.elementData;
                    if (offset + cursor >= elementData.length) {
                        throw new ConcurrentModificationException();
                    }
                    while (cursor != size && ArrayList.SubList.this.modCount == this.expectedModCount) {
                        consumer.accept(elementData[offset + cursor++]);
                    }
                    int n = cursor;
                    this.cursor = n;
                    this.lastRet = n;
                    this.checkForComodification();
                    return;
                }
                
                @Override
                public int nextIndex() {
                    return this.cursor;
                }
                
                @Override
                public int previousIndex() {
                    return this.cursor - 1;
                }
                
                @Override
                public void remove() {
                    if (this.lastRet < 0) {
                        throw new IllegalStateException();
                    }
                    this.checkForComodification();
                    try {
                        ArrayList.SubList.this.remove(this.lastRet);
                        this.cursor = this.lastRet;
                        this.lastRet = -1;
                        this.expectedModCount = ArrayList.this.modCount;
                    }
                    catch (IndexOutOfBoundsException ex) {
                        throw new ConcurrentModificationException();
                    }
                    return;
                }
                
                @Override
                public void set(E e) {
                    if (this.lastRet < 0) {
                        throw new IllegalStateException();
                    }
                    this.checkForComodification();
                    try {
                        ArrayList.this.set(offset + this.lastRet, e);
                    }
                    catch (IndexOutOfBoundsException ex) {
                        throw new ConcurrentModificationException();
                    }
                    return;
                }
                
                @Override
                public void add(E e) {
                    this.checkForComodification();
                    try {
                        int cursor = this.cursor;
                        ArrayList.SubList.this.add(cursor, e);
                        this.cursor = cursor + 1;
                        this.lastRet = -1;
                        this.expectedModCount = ArrayList.this.modCount;
                    }
                    catch (IndexOutOfBoundsException ex) {
                        throw new ConcurrentModificationException();
                    }
                }
                
                final void checkForComodification() {
                    if (this.expectedModCount != ArrayList.this.modCount) {
                        throw new ConcurrentModificationException();
                    }
                    return;
                }
            };
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            ArrayList.subListRangeCheck(n, n2, this.size);
            return new ArrayList.SubList(this, this.offset, n, n2);
        }
        
        private void rangeCheck(int n) {
            if (n < 0 || n >= this.size) {
                throw new IndexOutOfBoundsException(this.outOfBoundsMsg(n));
            }
            return;
        }
        
        private void rangeCheckForAdd(int n) {
            if (n < 0 || n > this.size) {
                throw new IndexOutOfBoundsException(this.outOfBoundsMsg(n));
            }
            return;
        }
        
        private String outOfBoundsMsg(int i) {
            return "Index: " + i + ", Size: " + this.size;
        }
        
        private void checkForComodification() {
            if (ArrayList.this.modCount != this.modCount) {
                throw new ConcurrentModificationException();
            }
            return;
        }
        
        @Override
        public Spliterator<E> spliterator() {
            this.checkForComodification();
            return new ArrayList.ArrayListSpliterator(ArrayList.this, this.offset, this.offset + this.size, this.modCount);
        }
    }
    
    static final class ArrayListSpliterator<E> implements Spliterator<E>
    {
        private final ArrayList<E> list;
        private int index;
        private int fence;
        private int expectedModCount;
        
        ArrayListSpliterator(ArrayList<E> list, int index, int fence, int expectedModCount) {
            this.list = list;
            this.index = index;
            this.fence = fence;
            this.expectedModCount = expectedModCount;
        }
        
        private int getFence() {
            int fence;
            if ((fence = this.fence) < 0) {
                ArrayList<Object> list;
                if ((list = this.list) == null) {
                    int fence2 = 0;
                    this.fence = fence2;
                    fence = fence2;
                }
                else {
                    this.expectedModCount = list.modCount;
                    int access$000 = list.size;
                    this.fence = access$000;
                    fence = access$000;
                }
            }
            return fence;
        }
        
        @Override
        public ArrayList.ArrayListSpliterator<E> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int index2 = index + fence >>> 1;
            return (index >= index2) ? null : new ArrayList.ArrayListSpliterator<E>(this.list, index, this.index = index2, this.expectedModCount);
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super E> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            int fence = this.getFence();
            int index = this.index;
            if (index < fence) {
                this.index = index + 1;
                consumer.accept(this.list.elementData[index]);
                if (this.list.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
                return true;
            }
            else {
                return false;
            }
        }
        
        @Override
        public void forEachRemaining(Consumer<? super E> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ArrayList<Object> list;
            Object[] elementData;
            if ((list = this.list) != null && (elementData = list.elementData) != null) {
                int index;
                int n;
                if ((index = this.fence) < 0) {
                    n = list.modCount;
                    index = list.size;
                }
                else {
                    n = this.expectedModCount;
                }
                int i;
                if ((i = this.index) >= 0 && (this.index = index) <= elementData.length) {
                    while (i < index) {
                        consumer.accept(elementData[i]);
                        ++i;
                    }
                    if (list.modCount == n) {
                        return;
                    }
                }
            }
            throw new ConcurrentModificationException();
        }
        
        @Override
        public long estimateSize() {
            return (long)(this.getFence() - this.index);
        }
        
        @Override
        public int characteristics() {
            return 16464;
        }
    }
}
